@@ -6,12 +6,12 @@ test_that(
       a = .1,
       b = 1 / (markov_cycle + 1)
     )
-    mat1 <- define_matrix(
+    mat1 <- define_transition(
       state_names = c("X1", "X2"),
       1-a, a,
       1-b, b
     )
-    mat2 <- define_matrix(
+    mat2 <- define_transition(
       state_names = c("X1", "X3"),
       1-a, a,
       1-b, b
@@ -25,7 +25,7 @@ test_that(
       y = 1726
     )
     
-    mod1 <- define_model(
+    mod1 <- define_strategy(
       transition_matrix = mat1,
       X1 = s1,
       X2 = s2
@@ -42,94 +42,101 @@ test_that(
       a = 456,
       b = 1029
     )
-    mod2 <- define_model(
+    mod2 <- define_strategy(
       transition_matrix = mat1,
       X1 = s3,
       X2 = s4
     )
-    mod3 <- define_model(
+    mod3 <- define_strategy(
       transition_matrix = mat2,
       X1 = s3,
       X3 = s4
     )
-    mod4 <- define_model(
+    mod4 <- define_strategy(
       transition_matrix = mat1,
       X1 = s5,
       X2 = s5
     )
     expect_error(
-      run_models(
+      run_model(
         mod1, mod2,
         parameters = par1,
         init = c(1, 2, 3)
       )
     )
     expect_error(
-      run_models(
+      run_model(
         mod1, mod2,
         parameters = par1,
         init = c(X3 = 1, X4 = 2)
       )
     )
     expect_error(
-      run_models(
+      run_model(
         mod1, mod2,
         parameters = par1,
         init = c(-1, 0)
       )
     )
     expect_error(
-      run_models(
+      run_model(
         mod1, mod2,
         parameters = par1,
         init = c(1, 0, 0)
       )
     )
     expect_error(
-      run_models(
+      run_model(
         mod1, mod2,
         parameters = par1,
         init = c(NA, 1)
       )
     )
     expect_error(
-      run_models(
+      run_model(
         mod1, mod2,
         parameters = par1,
         cycles = 0
       )
     )
     expect_error(
-      run_models(
+      run_model(
+        mod1, mod2,
+        parameters = par1,
+        cycles = 11.5
+      )
+    )
+    expect_error(
+      run_model(
         mod1,
         parameters = par1, list()
       )
     )
     expect_error(
-      run_models(
+      run_model(
         mod1, mod2,
         parameters = par1,
         method = "zzz"
       )
     )
     expect_error(
-      run_models(
+      run_model(
         mod1, mod3, parameters = par1
       )
     )
     expect_error(
-      run_models(
+      run_model(
         mod1, mod4, parameters = par1
       )
     )
     expect_warning(
-      run_models(
+      run_model(
         mod1, mod2,
         parameters = par1
       )
     )
     expect_warning(
-      run_models(
+      run_model(
         mod1, mod2,
         parameters = par1,
         cost = x
@@ -139,12 +146,12 @@ test_that(
 )
 
 test_that(
-  "run_models behaves as expected", {
+  "run_model behaves as expected", {
     par1 <- define_parameters(
       a = .1,
       b = 1 / (markov_cycle + 1)
     )
-    mat1 <- define_matrix(
+    mat1 <- define_transition(
       state_names = c("X1", "X2"),
       1-a, a,
       1-b, b
@@ -157,7 +164,7 @@ test_that(
       x = 987,
       y = 1726
     )
-    mod1 <- define_model(
+    mod1 <- define_strategy(
       transition_matrix = mat1,
       X1 = s1,
       X2 = s2
@@ -170,34 +177,34 @@ test_that(
       x = 456,
       y = 1029
     )
-    mod2 <- define_model(
+    mod2 <- define_strategy(
       transition_matrix = mat1,
       X1 = s3,
       X2 = s4
     )
     
     expect_identical(
-      run_models(mod1, mod2,
+      run_model(mod1, mod2,
                  parameters = par1, init = c(1000L, 0L), cost = x, effect = y),
-      run_models(mod1, mod2,
+      run_model(mod1, mod2,
                  parameters = par1, cost = x, effect = y)
     )
     expect_identical(
-      run_models(mod1, mod2,
+      run_model(mod1, mod2,
                  parameters = par1, cost = x, effect = y),
-      run_models(I = mod1, II = mod2,
+      run_model(I = mod1, II = mod2,
                  parameters = par1, cost = x, effect = y)
     )
     expect_warning(
-      run_models(I = mod1, mod2,
+      run_model(I = mod1, mod2,
                  parameters = par1, cost = x, effect = y)
     )
     expect_warning(
-      summary(run_models(mod1, mod2,
+      summary(run_model(mod1, mod2,
                          parameters = par1))
     )
     expect_output(
-      str(run_models(mod1, mod2,
+      str(run_model(mod1, mod2,
                      parameters = par1, cost = x, effect = y,
                      method = "beginning")),
       '2 obs. of  5 variables:
@@ -209,7 +216,7 @@ test_that(
       fixed = TRUE
     )
     s_mod <- summary(
-      run_models(
+      run_model(
         mod1, mod2,
         parameters = par1, cost = x, effect = y,
         method = "beginning"))
@@ -226,20 +233,21 @@ test_that(
       round(s_mod$res$y), c(283300, 891300)
     )
     
-    res_b <- run_models(mod1, mod2,
+    res_b <- run_model(mod1, mod2,
                         parameters = par1, cost = x, effect = y,
                         method = "beginning")
-    res_e <- run_models(mod1, mod2,
+    res_e <- run_model(mod1, mod2,
                         parameters = par1, cost = x, effect = y,
                         method = "end")
-    res_h <- run_models(mod1, mod2,
+    res_h <- run_model(mod1, mod2,
                         parameters = par1, cost = x, effect = y,
                         method = "half-cycle")
-    res_l <- run_models(mod1, mod2,
+    res_l <- run_model(mod1, mod2,
                         parameters = par1, cost = x, effect = y,
                         method = "life-table")
     
     plot(res_b, type = "counts")
+    plot(res_b, type = "values", value = c("x", "y"))
     plot(res_b, type = "ce")
     
     expect_output(
@@ -259,10 +267,30 @@ test_that(
       "II 688.8  680.5 1.012197"
     )
     expect_error(
-      run_models(mod1, mod2,
+      run_model(mod1, mod2,
                  parameters = par1, cost = x, effect = y,
                  method = "testtest")
     )
+    rm <- run_model(mod1, mod2,
+                     parameters = par1, cost = x, effect = y,
+                     cycles = 5)
+    expect_equivalent(
+      round(unlist(get_counts(rm, 1))),
+      c(950, 888, 879, 885, 890, 50, 112, 121, 115, 110)
+    )
+    expect_identical(
+      get_counts(rm, 1),
+      get_counts(rm, "I")
+    )
+    expect_equivalent(
+      get_init(rm),
+      c(1000, 0)
+    )
+    
+    expect_error(plot(rm, include_states = "C"))
+    
+    plot(rm, panels = "by_model")
+    plot(rm, panels = "by_state")
   }
 )
 
@@ -275,7 +303,7 @@ test_that("Discounting", {
     c2 = 876,
     c3 = 1726
   )
-  mat1 <- define_matrix(
+  mat1 <- define_transition(
     state_names = c("X1", "X2"),
     1-a, a,
     1-b, b
@@ -289,7 +317,7 @@ test_that("Discounting", {
     x = 987,
     y = 1726
   )
-  mod1 <- define_model(
+  mod1 <- define_strategy(
     transition_matrix = mat1,
     X1 = s1,
     X2 = s2
@@ -303,7 +331,7 @@ test_that("Discounting", {
     x = 456,
     y = 1029
   )
-  mod2 <- define_model(
+  mod2 <- define_strategy(
     transition_matrix = mat1,
     X1 = s3,
     X2 = s4
@@ -313,7 +341,7 @@ test_that("Discounting", {
     x = discount(c1, 0),
     y = discount(c3, 0)
   )
-  mod3 <- define_model(
+  mod3 <- define_strategy(
     transition_matrix = mat1,
     X1 = s1,
     X2 = s5
@@ -327,22 +355,22 @@ test_that("Discounting", {
     x = 456,
     y = 1029
   )
-  mod4 <- define_model(
+  mod4 <- define_strategy(
     transition_matrix = mat1,
     X1 = s6,
     X2 = s4
   )
-  res <- run_models(mod1, mod2, cycles = 10,
+  res <- run_model(mod1, mod2, cycles = 10,
                     parameters = par1, cost = x, effect = y,
                     method = "beginning")
   expect_output(
     print(res),
     "II 3292.352 4193.422 0.7851231"
   )
-  res1 <- run_models(mod1, mod2, cycles = 10,
+  res1 <- run_model(mod1, mod2, cycles = 10,
                      parameters = par1, cost = x, effect = y,
                      method = "beginning")
-  res2 <- run_models(mod3, mod2, cycles = 10,
+  res2 <- run_model(mod3, mod2, cycles = 10,
                      parameters = par1, cost = x, effect = y,
                      method = "beginning")
   expect_output(
@@ -357,7 +385,63 @@ II 6437001 7136374"
   )
   
   expect_error(
-    run_models(mod1, mod4, cycles = 10,
+    run_model(mod1, mod4, cycles = 10,
                parameters = par1, cost = x, effect = y)
   )
 })
+
+test_that(
+  "check_model_index works", {
+    
+    par1 <- define_parameters(
+      a = .1,
+      b = 1 / (markov_cycle + 1)
+    )
+    mat1 <- define_transition(
+      state_names = c("X1", "X2"),
+      1-a, a,
+      1-b, b
+    )
+    s1 <- define_state(
+      x = 234,
+      y = 123
+    )
+    s2 <- define_state(
+      x = 987,
+      y = 1726
+    )
+    mod1 <- define_strategy(
+      transition_matrix = mat1,
+      X1 = s1,
+      X2 = s2
+    )
+    s3 <- define_state(
+      x = 987,
+      y = 876
+    )
+    s4 <- define_state(
+      x = 456,
+      y = 1029
+    )
+    mod2 <- define_strategy(
+      transition_matrix = mat1,
+      X1 = s3,
+      X2 = s4
+    )
+    
+    res <- run_model(
+      mod1, mod2,
+      parameters = par1, init = c(1000L, 0L),
+      cost = x, effect = y)
+    
+    expect_error(
+      heemod:::check_model_index(res, 1:2)
+    )
+    expect_error(
+      heemod:::check_model_index(res, as.factor("I"))
+    )
+    expect_error(
+      heemod:::check_model_index(res, "a")
+    )
+  }
+)
